# Common Helm chart library

This chart is designed to make it easier for you to build and maintain Helm charts. It provides utilities that reflect best practices of Kubernetes chart development, making it faster for you to write charts.

## Install

Before you staring using the functions defined in this chart, you'll need to add it as a dependency.

1. Create a `requirements.yaml` file in the root directory of your chart.

    ```yaml
    commonlib_version: &clv 0.1.0

    appr:
      - name: quay.io/domino/helm-common
        repository: appr://
        version: *clv

    dependencies:
      - name: common
        repository: file://appr_charts/domino/helm-common
        version: *clv
    ```

2. Use `helm` to install this dependency.

    ```shell
    helm registry dep
    helm dependency build
    ```

## Template Functions

### `common.name`

`common.name` generates a name without any reference to the chart RELEASE. It is truncated at 63 chars because some Kubernetes name fields are limited to this. The variable `nameOverride` can be used to override the default behavior.

Example template:

```go
name: {{ include "common.name" . }}
```

Example output:

```yaml
---
# the default behavior for the "replicator" chart
name: replicator

---
# setting nameOverride: to "other-name"
name: other-name
```

### `common.fullname`

`common.fullname` generates a name suitable for the `name:` field in Kubernetes metadata. The output will be prefixed with the name of the chart RELEASE. The variable `fullnameOverride` can be used to override the default behavior.

Example template:

```go
metadata:
  name: {{ include "common.fullname" . }}
```

Example output:

```yaml
---
# for release "rls-123" of the "replicator" chart
metadata:
  name: rls-123-replicator

---
# setting fullnameOverride: to "1011-equals-eleven"
metadata:
  name: 1011-equals-eleven
```

### `common._rootname` (advanced)

`common._rootname` can be used to dynamically modifiy the behavior of both `common.name` and `common.fullname`. Helm does not perform any interpolation on the `values.yaml` file, so it is impossible to create a new value that derives from two others.

Example `values.yaml`:

```go
# this will NOT work
cloudProvider: aws
nameOverride: "{{ .Values.cloudProvider }}-{{ .Chart.Name }}"
```

Instead, re-define the function inside `templates/_helpers.tpl`:

```go
{{- define  "common._rootname" -}}
{{- printf "%s-%s" .Values.cloudProvider .Chart.Name -}}
{{- end -}}
```

Examples template:

```yaml
spec:
  containers:
    - name: {{ include "common.name" . }}
```

Example output for `cluster-autoscaler` chart with `cloudProvider` set to `"aws"`:

```yaml
spec:
  containers:
    - name: aws-cluster-autoscaler
```



### `common.chart`

`common.chart` prints the chart name and version, escaped to be legal as a Kubernetes label field.

Example template:

```go
{{ include "common.chart" . }}
```

Example output for chart `foo` with version `1.2.3-beta.55+1234`:

```yaml
foo-1.2.3-beta.55_1234
```

### `common.imagename`

`common.imagename` expects a Map with the follwing keys: `registry`, `repository`, `tag`.
It will use those values to build a fully-qualified reference to a container image.

Example `values.yaml`:

```yaml
image:
  registry: quay.io
  repository: domino/docker-image-prune
  tag: my-tag
```

Example template:

```go
spec:
  containers:
    - image: {{ include "common.imagename" .Values.image }}
```

Example output:

```yaml
spec:
  containers:
    - image: quay.io/domino/docker-image-prune:my-tag
```

### `common.labelize`

`common.labelize` generates a set of labels from a map. Values are quoted, keys are not.

Example template:

```go
{{- $myMap := dict "one" "1" "two" "2" "three" "3" -}}
annotations:
  {{- include "common.labels" $myMap | indent 2 }}
```

Example output:

```yaml
annotations:
  one: "1"
  two: "2"
  three: "3"
```

### `common.labels.standard`

`common.labels.standard` generates our standard set of metadata labels.

Example template:

```go
metadata:
  labels:
    {{ include "common.labels.standard" . | nindent 4 }}
```

Example output:

```yaml
metadata:
  labels:
    app.kubernetes.io/name: replicator
    helm.sh/chart: replicator-0.1.0
    app.kubernetes.io/instance: RELEASE-NAME
    app.kubernetes.io/managed-by: Tiller
```

### `common.labels.selector`

`common.labels.selector` generates labels appropriate for Pod selector clauses.

Example template:

```go
selector:
  {{- include "common.labels.selector" . | nindent 2 }}
```

Example output:

```yaml
selector:
  app.kubernetes.io/name: k8s-event-pump
  app.kubernetes.io/instance: RELEASE-NAME
```

### `common.labels.component`

`common.labels.component` generates the recommended label used to mark a role
that a piece may play in an application. This macro expects a string as input.

Example template:

```go
selector:
  {{- include "common.labels.selector" . | nindent 2 }}
  {{- include "common.labels.component" "master" | nindent 2 }}
```

Example output:

```yaml
selector:
  app.kubernetes.io/name: elasticsearch
  app.kubernetes.io/instance: RELEASE-NAME
  app.kubernetes.io/component: master
```

### `common.ingress.rules`

`common.ingress.rules` generates rules suitable for `Ingress` manifests. If not vhosts are provided, then this function will generate a single, path-based routing rule.  If one or more vhosts are present, then a rule will be generated for each at the same path.

Example `templates/ingress.yaml`:

```go
spec:
  rules:
    {{- include "common.ingress.rules" . | indent 4 }}
```

Example output for release `rls` of the `docker-registry` chart:

```yaml
# ingress.path set to "/"
---
# ingress.hosts is empty
spec:
  rules:
    - http:
        paths:
          - path: /
            backend:
              serviceName: rls-docker-registry
              servicePort: http

---
# ingress.hosts set to ["vhost1", "vhost2"]
spec:
  rules:
    - host: "vhost1"
      http:
        paths:
          - path: /
            backend:
              serviceName: rls-docker-registry
              servicePort: http
    - host: "vhost2"
      http:
        paths:
          - path: /
            backend:
              serviceName: rls-docker-registry
              servicePort: http
```


### `common.resources.deployment`

`common.resources.deployment` generates a resource definition of Deployment that is compatible with Kuberentes v1.5. This is required for running some services in current Domino deployments. As we target Kubernetes v1.12 and above, please do not use this unless your Helm chart will be deployed to a current customer.

Example output:
```yaml
# For Kubernetes >= v1.9
apiVersion: apps/v1
kind: Deployment

# For Kubernetes >= v1.7
apiVersion: apps/v1beta1
kind: Deployment

# All others
apiVersion: extensions/v1beta1
kind: Deployment
```


### common.joinList
`common.joinList` will "join" a list into a string with a given separator
(ie comma, pipe, etc.).

Let's say we have the following values:

```
values: |
  test:
    - foo
    - bar
```

We could call it with a comma separator:

```
  {{ tuple "," .Values.test | include "common.joinList" }}
```

And we'd get the following output:

```
  foo,bar
```
