{{/* vim: set filetype=mustache: */}}
{{/*
Defines the "root" name used for naming chart resources.
Provides consuming charts the ability to override the default value and still
retain the behavior of other, named-based functions in this library.
*/}}
{{- define "common._rootname" -}}
{{ .Chart.Name }}
{{- end -}}

{{/*
Expand the name of the chart.
*/}}
{{- define "common.name" -}}
{{- default (include "common._rootname" .) .Values.nameOverride | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Create a default fully qualified app name.
We truncate at 63 chars because some Kubernetes name fields are limited to this (by the DNS naming spec).
If release name contains chart name it will be used as a full name.
*/}}
{{- define "common.fullname" -}}
{{- if .Values.fullnameOverride -}}
{{- .Values.fullnameOverride | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- $name := default (include "common._rootname" .) .Values.nameOverride -}}
{{- if contains $name .Release.Name -}}
{{- .Release.Name | trunc 63 | trimSuffix "-" -}}
{{- else -}}
{{- printf "%s-%s" .Release.Name $name | trunc 63 | trimSuffix "-" -}}
{{- end -}}
{{- end -}}
{{- end -}}

{{/*
Create chart name and version as used by the chart label.
*/}}
{{- define "common.chart" -}}
{{- printf "%s-%s" .Chart.Name .Chart.Version | replace "+" "_" | trunc 63 | trimSuffix "-" -}}
{{- end -}}

{{/*
Create the name of the service account to use
*/}}
{{- define "common.serviceAccountName" -}}
{{- if .Values.serviceAccount.create -}}
    {{ default (include "common.fullname" .) .Values.serviceAccount.name }}
{{- else -}}
    {{ default "default" .Values.serviceAccount.name }}
{{- end -}}
{{- end -}}

{{/*
Generate fully-qualified reference to the container image.

Input should be map with the following values:

registry: Hostname for the docker registry (omit for dockerhub)
repository: Path to the docker image
tag: Docker image tag
*/}}
{{- define "common.imagename" -}}
{{- $imageName := printf "%s:%v" .repository .tag -}}
{{- if .registry -}}
{{- printf "%s/%s" (trimSuffix "/" .registry) $imageName -}}
{{- else -}}
{{- print $imageName -}}
{{- end -}}
{{- end -}}

{{/*
Takes a dict or map and generates labels. Values will be quoted, keys will not.
*/}}
{{- define "common.labelize" -}}
{{- range $k, $v := . }}
{{ $k }}: {{ $v | quote }}
{{- end -}}
{{- end -}}

{{/*
Creates a app version that is compatible with Kubernetes version
*/}}
{{- define "common.resources.app" -}}
{{- if semverCompare "^1.9-0" .Capabilities.KubeVersion.GitVersion }}
apiVersion: apps/v1
{{- else if semverCompare "^1.7-0" .Capabilities.KubeVersion.GitVersion }}
apiVersion: apps/v1beta1
{{- else }}
apiVersion: extensions/v1beta1
{{- end }}
{{- end -}}

{{/*
abstract: |
  Joins a list of prefixed values into a space seperated string
  Variation on the joins from here:
  https://github.com/openstack/openstack-helm-infra/tree/master/helm-toolkit/templates/utils
values: |
  test:
    - foo
    - bar
usage: |
  {{ tuple "(separator)" .Values.test | include "common.joinList" }}
return: |
  foo(separator)bar
*/}}
{{- define "common.joinList" -}}
{{- $sep := index . 0 -}}
{{- $local := dict "first" true -}}
{{- range $k, $v := index . 1 -}}{{- if not $local.first -}}{{- $sep -}}{{- end -}}{{- $v -}}{{- $_ := set $local "first" false -}}{{- end -}}
{{- end -}}
